from __future__ import unicode_literals

from django.apps import apps
from django.core.exceptions import ImproperlyConfigured
from django.db import models

from feature_flipper import flipper_settings


def get_feature_model():
    """Return the FeatureFlipper model defined in settings.py
    """
    try:
        return apps.get_model(flipper_settings.FEATURE_FLIPPER_MODEL)
    except ValueError:
        raise ImproperlyConfigured(
            "FEATURE_FLIPPER_MODEL must be of the form 'app_label.model_name'")
    except LookupError:
        raise ImproperlyConfigured(
            "FEATURE_FLIPPER_MODEL refers to model '{}' that has not been"
            " installed".format(flipper_settings.FEATURE_FLIPPER_MODEL))


class FeatureFlipperQuerySet(models.query.QuerySet):
    """Provides a handler to look up feature flags for the given configuration.
    """
    def get_feature(self, feature):
        """Return the feature mapping.
        """
        return self.filter(feature=feature)

    def show_feature(self, user, feature):
        """Return True or False for the given feature.
        """
        user_filter = {
            self.model.USER_FEATURE_FIELD: user,
        }
        return self.get_feature(feature).filter(
            models.Q(**user_filter) | models.Q(everyone=True)).exists()


class AbstractFeatureFlipper(models.Model):
    """The basic feature flipper data that gets shared among all other
    applications
    """
    objects = FeatureFlipperQuerySet.as_manager()

    feature = models.CharField(
        max_length=15, choices=flipper_settings.FEATURE_FLIPPER_FLAGS)
    everyone = models.BooleanField(default=False)

    class Meta:
        abstract = True


class FeatureFlipper(AbstractFeatureFlipper):
    """A feature flipper hooked up to the Django User referenced by the user's
    settings. This provides a simple flipper class that most developers can use
    out of the box
    """
    USER_FEATURE_FIELD = 'user'

    user = models.ForeignKey(flipper_settings.AUTH_USER_MODEL)

    class Meta:
        swappable = 'FEATURE_FLIPPER_MODEL'



import string

from django import template

from ..models import get_feature_model


register = template.Library()


def show_feature(item, feature):
    """Return True/False whether the assigned feature can be displayed. This is
    primarily used in the template tag to determine whether to render the
    content inside itself.
    """
    FeatureFlipper = get_feature_model()
    return FeatureFlipper.objects.show_feature(item, feature)


@register.tag(name='flipper')
def do_flipper(parser, token):
    """The flipper tag takes two arguments: the user to look up and the feature
    to compare against.
    """
    nodelist = parser.parse(('endflipper',))
    tag_name, user_key, feature = token.split_contents()
    parser.delete_first_token()
    return FlipperNode(nodelist, user_key, feature)


class FlipperNode(template.Node):
    """Handles rendering the Feature Flipper guarded content.
    """
    def __init__(self, nodelist, user_key, feature):
        """Setup the tag
        """
        self.nodelist = nodelist
        self.user_key = user_key
        self.feature = feature

    def render(self, context):
        """Handle the actual rendering.
        """
        user = self._get_value(self.user_key, context)
        feature = self._get_value(self.feature, context)
        allowed = show_feature(user, feature)

        return self.nodelist.render(context) if allowed else ''

    def _get_value(self, key, context):
        """Works out whether key is a value or if it's a variable referencing a
        value in context and returns the correct value.
        """
        string_quotes = ('"', "'")

        if key[0] in string_quotes and key[-1] in string_quotes:
            return key[1:-1]
        if key in string.digits:
            return int(key)
        return context[key]



from django.conf import settings

_DEFAULT_FEATURE_FLAGS = ()

FEATURE_FLIPPER_FLAGS = getattr(
    settings, 'FEATURE_FLIPPER_FLAGS',
    _DEFAULT_FEATURE_FLAGS)

AUTH_USER_MODEL = settings.AUTH_USER_MODEL

FEATURE_FLIPPER_MODEL = getattr(
    settings, 'FEATURE_FLIPPER_MODEL',
    'feature_flipper.FeatureFlipper')



# -*- coding: utf-8 -*-
# Generated by Django 1.9.8 on 2016-07-21 15:34
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='FeatureFlipper',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('feature', models.CharField(choices=[(b'feature_for_all', 'Feature for everyone'), (b'restricted_feature', 'Restricted Feature')], max_length=15)),
                ('everyone', models.BooleanField(default=False)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'swappable': 'FEATURE_FLIPPER_MODEL',
            },
        ),
    ]



from setuptools import find_packages, setup

try:
    with open('README.rst') as f:
        LONG_DESCRIPTION = f.read()
except:
    LONG_DESCRIPTION = ''

setup(
    name='django-feature-flipper',
    author='Pebble',
    author_email='scott.walton@mypebble.co.uk',
    description='A simple, customisable, feature flipper',
    url='https://github.com/mypebble/django-feature-flipper.git',
    version='0.0.3',
    packages=find_packages(),
    license='MIT',
    long_description=LONG_DESCRIPTION,
)
